# This code parses date/times, so please
#
#     pip install python-dateutil
#
# To use this code, make sure you
#
#     import json
#
# and then, to convert JSON from a string, do
#
#     result = route_parameters_from_dict(json.loads(json_string))

from enum import Enum
from typing import Optional, List, Any, TypeVar, Callable, Type, cast
from datetime import datetime
import dateutil.parser


T = TypeVar("T")
EnumT = TypeVar("EnumT", bound=Enum)


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def from_datetime(x: Any) -> datetime:
    return dateutil.parser.parse(x)


def to_enum(c: Type[EnumT], x: Any) -> EnumT:
    assert isinstance(x, c)
    return x.value


def from_float(x: Any) -> float:
    assert isinstance(x, (float, int)) and not isinstance(x, bool)
    return float(x)


def to_float(x: Any) -> float:
    assert isinstance(x, float)
    return x


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class Property(Enum):
    DISTANCE = "distance"
    ROUTE = "route"
    TRAVEL_TIME = "travel_time"


class Search:
    id: Optional[str]
    departure_location_id: Optional[str]
    arrival_location_ids: Optional[List[str]]
    transportation: Optional[str]
    departure_time: Optional[datetime]
    properties: Optional[List[Property]]

    def __init__(self, id: Optional[str], departure_location_id: Optional[str], arrival_location_ids: Optional[List[str]], transportation: Optional[str], departure_time: Optional[datetime], properties: Optional[List[Property]]) -> None:
        self.id = id
        self.departure_location_id = departure_location_id
        self.arrival_location_ids = arrival_location_ids
        self.transportation = transportation
        self.departure_time = departure_time
        self.properties = properties

    @staticmethod
    def from_dict(obj: Any) -> 'Search':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        departure_location_id = from_union([from_str, from_none], obj.get("departure_location_id"))
        arrival_location_ids = from_union([lambda x: from_list(from_str, x), from_none], obj.get("arrival_location_ids"))
        transportation = from_union([from_str, from_none], obj.get("transportation"))
        departure_time = from_union([from_datetime, from_none], obj.get("departure_time"))
        properties = from_union([lambda x: from_list(Property, x), from_none], obj.get("properties"))
        return Search(id, departure_location_id, arrival_location_ids, transportation, departure_time, properties)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["departure_location_id"] = from_union([from_str, from_none], self.departure_location_id)
        result["arrival_location_ids"] = from_union([lambda x: from_list(from_str, x), from_none], self.arrival_location_ids)
        result["transportation"] = from_union([from_str, from_none], self.transportation)
        result["departure_time"] = from_union([lambda x: x.isoformat(), from_none], self.departure_time)
        result["properties"] = from_union([lambda x: from_list(lambda x: to_enum(Property, x), x), from_none], self.properties)
        return result


class Coords:
    lng: Optional[float]
    lat: Optional[float]

    def __init__(self, lng: Optional[float], lat: Optional[float]) -> None:
        self.lng = lng
        self.lat = lat

    @staticmethod
    def from_dict(obj: Any) -> 'Coords':
        assert isinstance(obj, dict)
        lng = from_union([from_float, from_none], obj.get("lng"))
        lat = from_union([from_float, from_none], obj.get("lat"))
        return Coords(lng, lat)

    def to_dict(self) -> dict:
        result: dict = {}
        result["lng"] = from_union([to_float, from_none], self.lng)
        result["lat"] = from_union([to_float, from_none], self.lat)
        return result


class Location:
    id: Optional[str]
    coords: Optional[Coords]

    def __init__(self, id: Optional[str], coords: Optional[Coords]) -> None:
        self.id = id
        self.coords = coords

    @staticmethod
    def from_dict(obj: Any) -> 'Location':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        coords = from_union([Coords.from_dict, from_none], obj.get("coords"))
        return Location(id, coords)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["coords"] = from_union([lambda x: to_class(Coords, x), from_none], self.coords)
        return result


class RouteParameter:
    locations: Optional[List[Location]]
    departure_searches: Optional[List[Search]]
    arrival_searches: Optional[List[Search]]

    def __init__(self, locations: Optional[List[Location]], departure_searches: Optional[List[Search]], arrival_searches: Optional[List[Search]]) -> None:
        self.locations = locations
        self.departure_searches = departure_searches
        self.arrival_searches = arrival_searches

    @staticmethod
    def from_dict(obj: Any) -> 'RouteParameter':
        assert isinstance(obj, dict)
        locations = from_union([lambda x: from_list(Location.from_dict, x), from_none], obj.get("locations"))
        departure_searches = from_union([lambda x: from_list(Search.from_dict, x), from_none], obj.get("departure_searches"))
        arrival_searches = from_union([lambda x: from_list(Search.from_dict, x), from_none], obj.get("arrival_searches"))
        return RouteParameter(locations, departure_searches, arrival_searches)

    def to_dict(self) -> dict:
        result: dict = {}
        result["locations"] = from_union([lambda x: from_list(lambda x: to_class(Location, x), x), from_none], self.locations)
        result["departure_searches"] = from_union([lambda x: from_list(lambda x: to_class(Search, x), x), from_none], self.departure_searches)
        result["arrival_searches"] = from_union([lambda x: from_list(lambda x: to_class(Search, x), x), from_none], self.arrival_searches)
        return result


def route_parameters_from_dict(s: Any) -> List[RouteParameter]:
    return from_list(RouteParameter.from_dict, s)


def route_parameters_to_dict(x: List[RouteParameter]) -> Any:
    return from_list(lambda x: to_class(RouteParameter, x), x)
