# This code parses date/times, so please
#
#     pip install python-dateutil
#
# To use this code, make sure you
#
#     import json
#
# and then, to convert JSON from a string, do
#
#     result = time_map_param_from_dict(json.loads(json_string))

from typing import Optional, Any, List, TypeVar, Type, cast, Callable
from datetime import datetime
import dateutil.parser


T = TypeVar("T")


def from_float(x: Any) -> float:
    assert isinstance(x, (float, int)) and not isinstance(x, bool)
    return float(x)


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def to_float(x: Any) -> float:
    assert isinstance(x, float)
    return x


def from_bool(x: Any) -> bool:
    assert isinstance(x, bool)
    return x


def from_int(x: Any) -> int:
    assert isinstance(x, int) and not isinstance(x, bool)
    return x


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_datetime(x: Any) -> datetime:
    return dateutil.parser.parse(x)


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


class Coords:
    lat: Optional[float]
    lng: Optional[float]

    def __init__(self, lat: Optional[float], lng: Optional[float]) -> None:
        self.lat = lat
        self.lng = lng

    @staticmethod
    def from_dict(obj: Any) -> 'Coords':
        assert isinstance(obj, dict)
        lat = from_union([from_float, from_none], obj.get("lat"))
        lng = from_union([from_float, from_none], obj.get("lng"))
        return Coords(lat, lng)

    def to_dict(self) -> dict:
        result: dict = {}
        result["lat"] = from_union([to_float, from_none], self.lat)
        result["lng"] = from_union([to_float, from_none], self.lng)
        return result


class Range:
    enabled: Optional[bool]
    width: Optional[int]

    def __init__(self, enabled: Optional[bool], width: Optional[int]) -> None:
        self.enabled = enabled
        self.width = width

    @staticmethod
    def from_dict(obj: Any) -> 'Range':
        assert isinstance(obj, dict)
        enabled = from_union([from_bool, from_none], obj.get("enabled"))
        width = from_union([from_int, from_none], obj.get("width"))
        return Range(enabled, width)

    def to_dict(self) -> dict:
        result: dict = {}
        result["enabled"] = from_union([from_bool, from_none], self.enabled)
        result["width"] = from_union([from_int, from_none], self.width)
        return result


class Transportation:
    type: Optional[str]

    def __init__(self, type: Optional[str]) -> None:
        self.type = type

    @staticmethod
    def from_dict(obj: Any) -> 'Transportation':
        assert isinstance(obj, dict)
        type = from_union([from_str, from_none], obj.get("type"))
        return Transportation(type)

    def to_dict(self) -> dict:
        result: dict = {}
        result["type"] = from_union([from_str, from_none], self.type)
        return result


class ArrivalSearch:
    id: Optional[str]
    coords: Optional[Coords]
    transportation: Optional[Transportation]
    arrival_time: Optional[datetime]
    travel_time: Optional[int]
    range: Optional[Range]

    def __init__(self, id: Optional[str], coords: Optional[Coords], transportation: Optional[Transportation], arrival_time: Optional[datetime], travel_time: Optional[int], range: Optional[Range]) -> None:
        self.id = id
        self.coords = coords
        self.transportation = transportation
        self.arrival_time = arrival_time
        self.travel_time = travel_time
        self.range = range

    @staticmethod
    def from_dict(obj: Any) -> 'ArrivalSearch':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        coords = from_union([Coords.from_dict, from_none], obj.get("coords"))
        transportation = from_union([Transportation.from_dict, from_none], obj.get("transportation"))
        arrival_time = from_union([from_datetime, from_none], obj.get("arrival_time"))
        travel_time = from_union([from_int, from_none], obj.get("travel_time"))
        range = from_union([Range.from_dict, from_none], obj.get("range"))
        return ArrivalSearch(id, coords, transportation, arrival_time, travel_time, range)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["coords"] = from_union([lambda x: to_class(Coords, x), from_none], self.coords)
        result["transportation"] = from_union([lambda x: to_class(Transportation, x), from_none], self.transportation)
        result["arrival_time"] = from_union([lambda x: x.isoformat(), from_none], self.arrival_time)
        result["travel_time"] = from_union([from_int, from_none], self.travel_time)
        result["range"] = from_union([lambda x: to_class(Range, x), from_none], self.range)
        return result


class DepartureSearch:
    id: Optional[str]
    coords: Optional[Coords]
    transportation: Optional[Transportation]
    departure_time: Optional[datetime]
    travel_time: Optional[int]

    def __init__(self, id: Optional[str], coords: Optional[Coords], transportation: Optional[Transportation], departure_time: Optional[datetime], travel_time: Optional[int]) -> None:
        self.id = id
        self.coords = coords
        self.transportation = transportation
        self.departure_time = departure_time
        self.travel_time = travel_time

    @staticmethod
    def from_dict(obj: Any) -> 'DepartureSearch':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        coords = from_union([Coords.from_dict, from_none], obj.get("coords"))
        transportation = from_union([Transportation.from_dict, from_none], obj.get("transportation"))
        departure_time = from_union([from_datetime, from_none], obj.get("departure_time"))
        travel_time = from_union([from_int, from_none], obj.get("travel_time"))
        return DepartureSearch(id, coords, transportation, departure_time, travel_time)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["coords"] = from_union([lambda x: to_class(Coords, x), from_none], self.coords)
        result["transportation"] = from_union([lambda x: to_class(Transportation, x), from_none], self.transportation)
        result["departure_time"] = from_union([lambda x: x.isoformat(), from_none], self.departure_time)
        result["travel_time"] = from_union([from_int, from_none], self.travel_time)
        return result


class Ion:
    id: Optional[str]
    search_ids: Optional[List[str]]

    def __init__(self, id: Optional[str], search_ids: Optional[List[str]]) -> None:
        self.id = id
        self.search_ids = search_ids

    @staticmethod
    def from_dict(obj: Any) -> 'Ion':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        search_ids = from_union([lambda x: from_list(from_str, x), from_none], obj.get("search_ids"))
        return Ion(id, search_ids)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["search_ids"] = from_union([lambda x: from_list(from_str, x), from_none], self.search_ids)
        return result


class TimeMapParams:
    departure_searches: Optional[List[DepartureSearch]]
    arrival_searches: Optional[List[ArrivalSearch]]
    unions: Optional[List[Ion]]
    intersections: Optional[List[Ion]]

    def __init__(self, departure_searches: Optional[List[DepartureSearch]], arrival_searches: Optional[List[ArrivalSearch]], unions: Optional[List[Ion]], intersections: Optional[List[Ion]]) -> None:
        self.departure_searches = departure_searches
        self.arrival_searches = arrival_searches
        self.unions = unions
        self.intersections = intersections

    @staticmethod
    def from_dict(obj: Any) -> 'TimeMapParams':
        assert isinstance(obj, dict)
        departure_searches = from_union([lambda x: from_list(DepartureSearch.from_dict, x), from_none], obj.get("departure_searches"))
        arrival_searches = from_union([lambda x: from_list(ArrivalSearch.from_dict, x), from_none], obj.get("arrival_searches"))
        unions = from_union([lambda x: from_list(Ion.from_dict, x), from_none], obj.get("unions"))
        intersections = from_union([lambda x: from_list(Ion.from_dict, x), from_none], obj.get("intersections"))
        return TimeMapParams(departure_searches, arrival_searches, unions, intersections)

    def to_dict(self) -> dict:
        result: dict = {}
        result["departure_searches"] = from_union([lambda x: from_list(lambda x: to_class(DepartureSearch, x), x), from_none], self.departure_searches)
        result["arrival_searches"] = from_union([lambda x: from_list(lambda x: to_class(ArrivalSearch, x), x), from_none], self.arrival_searches)
        result["unions"] = from_union([lambda x: from_list(lambda x: to_class(Ion, x), x), from_none], self.unions)
        result["intersections"] = from_union([lambda x: from_list(lambda x: to_class(Ion, x), x), from_none], self.intersections)
        return result


def time_map_param_from_dict(s: Any) -> TimeMapParams:
    return TimeMapParams.from_dict(s)


def time_map_param_to_dict(x: TimeMapParams) -> Any:
    return to_class(TimeMapParams, x)
