# To use this code, make sure you
#
#     import json
#
# and then, to convert JSON from a string, do
#
#     result = time_filter_response_param_from_dict(json.loads(json_string))

from typing import Optional, Any, List, TypeVar, Callable, Type, cast


T = TypeVar("T")


def from_int(x: Any) -> int:
    assert isinstance(x, int) and not isinstance(x, bool)
    return x


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class Property:
    travel_time: Optional[int]

    def __init__(self, travel_time: Optional[int]) -> None:
        self.travel_time = travel_time

    @staticmethod
    def from_dict(obj: Any) -> 'Property':
        assert isinstance(obj, dict)
        travel_time = from_union([from_int, from_none], obj.get("travel_time"))
        return Property(travel_time)

    def to_dict(self) -> dict:
        result: dict = {}
        result["travel_time"] = from_union([from_int, from_none], self.travel_time)
        return result


class Location:
    id: Optional[str]
    properties: Optional[List[Property]]

    def __init__(self, id: Optional[str], properties: Optional[List[Property]]) -> None:
        self.id = id
        self.properties = properties

    @staticmethod
    def from_dict(obj: Any) -> 'Location':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        properties = from_union([lambda x: from_list(Property.from_dict, x), from_none], obj.get("properties"))
        return Location(id, properties)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["properties"] = from_union([lambda x: from_list(lambda x: to_class(Property, x), x), from_none], self.properties)
        return result


class Result:
    search_id: Optional[str]
    locations: Optional[List[Location]]
    unreachable: Optional[List[str]]

    def __init__(self, search_id: Optional[str], locations: Optional[List[Location]], unreachable: Optional[List[str]]) -> None:
        self.search_id = search_id
        self.locations = locations
        self.unreachable = unreachable

    @staticmethod
    def from_dict(obj: Any) -> 'Result':
        assert isinstance(obj, dict)
        search_id = from_union([from_str, from_none], obj.get("search_id"))
        locations = from_union([lambda x: from_list(Location.from_dict, x), from_none], obj.get("locations"))
        unreachable = from_union([lambda x: from_list(from_str, x), from_none], obj.get("unreachable"))
        return Result(search_id, locations, unreachable)

    def to_dict(self) -> dict:
        result: dict = {}
        result["search_id"] = from_union([from_str, from_none], self.search_id)
        result["locations"] = from_union([lambda x: from_list(lambda x: to_class(Location, x), x), from_none], self.locations)
        result["unreachable"] = from_union([lambda x: from_list(from_str, x), from_none], self.unreachable)
        return result


class TimeFilterResponseParam:
    results: Optional[List[Result]]

    def __init__(self, results: Optional[List[Result]]) -> None:
        self.results = results

    @staticmethod
    def from_dict(obj: Any) -> 'TimeFilterResponseParam':
        assert isinstance(obj, dict)
        results = from_union([lambda x: from_list(Result.from_dict, x), from_none], obj.get("results"))
        return TimeFilterResponseParam(results)

    def to_dict(self) -> dict:
        result: dict = {}
        result["results"] = from_union([lambda x: from_list(lambda x: to_class(Result, x), x), from_none], self.results)
        return result


def time_filter_response_param_from_dict(s: Any) -> TimeFilterResponseParam:
    return TimeFilterResponseParam.from_dict(s)


def time_filter_response_param_to_dict(x: TimeFilterResponseParam) -> Any:
    return to_class(TimeFilterResponseParam, x)
