# This code parses date/times, so please
#
#     pip install python-dateutil
#
# To use this code, make sure you
#
#     import json
#
# and then, to convert JSON from a string, do
#
#     result = response_parameters_from_dict(json.loads(json_string))

from typing import Optional, Any, List, TypeVar, Callable, Type, cast
from enum import Enum
from datetime import datetime
import dateutil.parser


T = TypeVar("T")
EnumT = TypeVar("EnumT", bound=Enum)


def from_float(x: Any) -> float:
    assert isinstance(x, (float, int)) and not isinstance(x, bool)
    return float(x)


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def to_float(x: Any) -> float:
    assert isinstance(x, float)
    return x


def from_int(x: Any) -> int:
    assert isinstance(x, int) and not isinstance(x, bool)
    return x


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_enum(c: Type[EnumT], x: Any) -> EnumT:
    assert isinstance(x, c)
    return x.value


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


def from_datetime(x: Any) -> datetime:
    return dateutil.parser.parse(x)


class Coord:
    lat: Optional[float]
    lng: Optional[float]

    def __init__(self, lat: Optional[float], lng: Optional[float]) -> None:
        self.lat = lat
        self.lng = lng

    @staticmethod
    def from_dict(obj: Any) -> 'Coord':
        assert isinstance(obj, dict)
        lat = from_union([from_float, from_none], obj.get("lat"))
        lng = from_union([from_float, from_none], obj.get("lng"))
        return Coord(lat, lng)

    def to_dict(self) -> dict:
        result: dict = {}
        result["lat"] = from_union([to_float, from_none], self.lat)
        result["lng"] = from_union([to_float, from_none], self.lng)
        return result


class Mode(Enum):
    BUS = "bus"
    RAIL_NATIONAL = "rail_national"
    RAIL_OVERGROUND = "rail_overground"
    RAIL_UNDERGROUND = "rail_underground"
    WALK = "walk"


class Turn(Enum):
    LEFT = "left"
    RIGHT = "right"
    SLIGHT_LEFT = "slight_left"
    SLIGHT_RIGHT = "slight_right"
    STRAIGHT = "straight"


class TypeEnum(Enum):
    BASIC = "basic"
    PUBLIC_TRANSPORT = "public_transport"
    ROAD = "road"
    START_END = "start_end"


class Part:
    id: Optional[int]
    type: Optional[TypeEnum]
    mode: Optional[Mode]
    directions: Optional[str]
    distance: Optional[int]
    travel_time: Optional[int]
    coords: Optional[List[Coord]]
    direction: Optional[str]
    road: Optional[str]
    turn: Optional[Turn]
    line: Optional[str]
    departure_station: Optional[str]
    arrival_station: Optional[str]
    departs_at: Optional[str]
    arrives_at: Optional[str]
    num_stops: Optional[int]

    def __init__(self, id: Optional[int], type: Optional[TypeEnum], mode: Optional[Mode], directions: Optional[str], distance: Optional[int], travel_time: Optional[int], coords: Optional[List[Coord]], direction: Optional[str], road: Optional[str], turn: Optional[Turn], line: Optional[str], departure_station: Optional[str], arrival_station: Optional[str], departs_at: Optional[str], arrives_at: Optional[str], num_stops: Optional[int]) -> None:
        self.id = id
        self.type = type
        self.mode = mode
        self.directions = directions
        self.distance = distance
        self.travel_time = travel_time
        self.coords = coords
        self.direction = direction
        self.road = road
        self.turn = turn
        self.line = line
        self.departure_station = departure_station
        self.arrival_station = arrival_station
        self.departs_at = departs_at
        self.arrives_at = arrives_at
        self.num_stops = num_stops

    @staticmethod
    def from_dict(obj: Any) -> 'Part':
        assert isinstance(obj, dict)
        id = from_union([from_int, from_none], obj.get("id"))
        type = from_union([TypeEnum, from_none], obj.get("type"))
        mode = from_union([Mode, from_none], obj.get("mode"))
        directions = from_union([from_str, from_none], obj.get("directions"))
        distance = from_union([from_int, from_none], obj.get("distance"))
        travel_time = from_union([from_int, from_none], obj.get("travel_time"))
        coords = from_union([lambda x: from_list(Coord.from_dict, x), from_none], obj.get("coords"))
        direction = from_union([from_str, from_none], obj.get("direction"))
        road = from_union([from_str, from_none], obj.get("road"))
        turn = from_union([Turn, from_none], obj.get("turn"))
        line = from_union([from_str, from_none], obj.get("line"))
        departure_station = from_union([from_str, from_none], obj.get("departure_station"))
        arrival_station = from_union([from_str, from_none], obj.get("arrival_station"))
        departs_at = from_union([from_str, from_none], obj.get("departs_at"))
        arrives_at = from_union([from_str, from_none], obj.get("arrives_at"))
        num_stops = from_union([from_int, from_none], obj.get("num_stops"))
        return Part(id, type, mode, directions, distance, travel_time, coords, direction, road, turn, line, departure_station, arrival_station, departs_at, arrives_at, num_stops)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_int, from_none], self.id)
        result["type"] = from_union([lambda x: to_enum(TypeEnum, x), from_none], self.type)
        result["mode"] = from_union([lambda x: to_enum(Mode, x), from_none], self.mode)
        result["directions"] = from_union([from_str, from_none], self.directions)
        result["distance"] = from_union([from_int, from_none], self.distance)
        result["travel_time"] = from_union([from_int, from_none], self.travel_time)
        result["coords"] = from_union([lambda x: from_list(lambda x: to_class(Coord, x), x), from_none], self.coords)
        result["direction"] = from_union([from_str, from_none], self.direction)
        result["road"] = from_union([from_str, from_none], self.road)
        result["turn"] = from_union([lambda x: to_enum(Turn, x), from_none], self.turn)
        result["line"] = from_union([from_str, from_none], self.line)
        result["departure_station"] = from_union([from_str, from_none], self.departure_station)
        result["arrival_station"] = from_union([from_str, from_none], self.arrival_station)
        result["departs_at"] = from_union([from_str, from_none], self.departs_at)
        result["arrives_at"] = from_union([from_str, from_none], self.arrives_at)
        result["num_stops"] = from_union([from_int, from_none], self.num_stops)
        return result


class Route:
    departure_time: Optional[datetime]
    arrival_time: Optional[datetime]
    parts: Optional[List[Part]]

    def __init__(self, departure_time: Optional[datetime], arrival_time: Optional[datetime], parts: Optional[List[Part]]) -> None:
        self.departure_time = departure_time
        self.arrival_time = arrival_time
        self.parts = parts

    @staticmethod
    def from_dict(obj: Any) -> 'Route':
        assert isinstance(obj, dict)
        departure_time = from_union([from_datetime, from_none], obj.get("departure_time"))
        arrival_time = from_union([from_datetime, from_none], obj.get("arrival_time"))
        parts = from_union([lambda x: from_list(Part.from_dict, x), from_none], obj.get("parts"))
        return Route(departure_time, arrival_time, parts)

    def to_dict(self) -> dict:
        result: dict = {}
        result["departure_time"] = from_union([lambda x: x.isoformat(), from_none], self.departure_time)
        result["arrival_time"] = from_union([lambda x: x.isoformat(), from_none], self.arrival_time)
        result["parts"] = from_union([lambda x: from_list(lambda x: to_class(Part, x), x), from_none], self.parts)
        return result


class Property:
    travel_time: Optional[int]
    distance: Optional[int]
    route: Optional[Route]

    def __init__(self, travel_time: Optional[int], distance: Optional[int], route: Optional[Route]) -> None:
        self.travel_time = travel_time
        self.distance = distance
        self.route = route

    @staticmethod
    def from_dict(obj: Any) -> 'Property':
        assert isinstance(obj, dict)
        travel_time = from_union([from_int, from_none], obj.get("travel_time"))
        distance = from_union([from_int, from_none], obj.get("distance"))
        route = from_union([Route.from_dict, from_none], obj.get("route"))
        return Property(travel_time, distance, route)

    def to_dict(self) -> dict:
        result: dict = {}
        result["travel_time"] = from_union([from_int, from_none], self.travel_time)
        result["distance"] = from_union([from_int, from_none], self.distance)
        result["route"] = from_union([lambda x: to_class(Route, x), from_none], self.route)
        return result


class Location:
    id: Optional[str]
    properties: Optional[List[Property]]

    def __init__(self, id: Optional[str], properties: Optional[List[Property]]) -> None:
        self.id = id
        self.properties = properties

    @staticmethod
    def from_dict(obj: Any) -> 'Location':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        properties = from_union([lambda x: from_list(Property.from_dict, x), from_none], obj.get("properties"))
        return Location(id, properties)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["properties"] = from_union([lambda x: from_list(lambda x: to_class(Property, x), x), from_none], self.properties)
        return result


class Result:
    search_id: Optional[str]
    locations: Optional[List[Location]]
    unreachable: Optional[List[Any]]

    def __init__(self, search_id: Optional[str], locations: Optional[List[Location]], unreachable: Optional[List[Any]]) -> None:
        self.search_id = search_id
        self.locations = locations
        self.unreachable = unreachable

    @staticmethod
    def from_dict(obj: Any) -> 'Result':
        assert isinstance(obj, dict)
        search_id = from_union([from_str, from_none], obj.get("search_id"))
        locations = from_union([lambda x: from_list(Location.from_dict, x), from_none], obj.get("locations"))
        unreachable = from_union([lambda x: from_list(lambda x: x, x), from_none], obj.get("unreachable"))
        return Result(search_id, locations, unreachable)

    def to_dict(self) -> dict:
        result: dict = {}
        result["search_id"] = from_union([from_str, from_none], self.search_id)
        result["locations"] = from_union([lambda x: from_list(lambda x: to_class(Location, x), x), from_none], self.locations)
        result["unreachable"] = from_union([lambda x: from_list(lambda x: x, x), from_none], self.unreachable)
        return result


class ResponseParameters:
    results: Optional[List[Result]]

    def __init__(self, results: Optional[List[Result]]) -> None:
        self.results = results

    @staticmethod
    def from_dict(obj: Any) -> 'ResponseParameters':
        assert isinstance(obj, dict)
        results = from_union([lambda x: from_list(Result.from_dict, x), from_none], obj.get("results"))
        return ResponseParameters(results)

    def to_dict(self) -> dict:
        result: dict = {}
        result["results"] = from_union([lambda x: from_list(lambda x: to_class(Result, x), x), from_none], self.results)
        return result


def response_parameters_from_dict(s: Any) -> ResponseParameters:
    return ResponseParameters.from_dict(s)


def response_parameters_to_dict(x: ResponseParameters) -> Any:
    return to_class(ResponseParameters, x)
